@@ -29,8 +29,10 @@ import (
 )
 
 var cmdDebug = &cobra.Command{
-	Use:   "debug",
-	Short: "Debug commands",
+	Use:               "debug",
+	Short:             "Debug commands",
+	GroupID:           cmdGroupDefault,
+	DisableAutoGenTag: true,
 }
 
 var cmdDebugDump = &cobra.Command{
@@ -47,6 +49,7 @@ Exit status is 0 if the command was successful.
 Exit status is 1 if there was any error.
 Exit status is 10 if the repository does not exist.
 Exit status is 11 if the repository is already locked.
+Exit status is 12 if the password is incorrect.
 `,
 	DisableAutoGenTag: true,
 	RunE: func(cmd *cobra.Command, args []string) error {
