@@ -5,8 +5,10 @@ import (
 )
 
 var cmdRepair = &cobra.Command{
-	Use:   "repair",
-	Short: "Repair the repository",
+	Use:               "repair",
+	Short:             "Repair the repository",
+	GroupID:           cmdGroupDefault,
+	DisableAutoGenTag: true,
 }
 
 func init() {
