@@ -4,20 +4,20 @@ import (
 	"context"
 	"fmt"
 	"runtime"
+	"strings"
 	"sync"
 	"sync/atomic"
 	"testing"
 
 	"github.com/restic/restic/internal/errors"
-	"github.com/restic/restic/internal/index"
 	"github.com/restic/restic/internal/restic"
+	rtest "github.com/restic/restic/internal/test"
 	"golang.org/x/sync/errgroup"
 )
 
 var errTest = errors.New("test error")
 
 type saveFail struct {
-	idx    restic.MasterIndex
 	cnt    int32
 	failAt int32
 }
@@ -31,18 +31,12 @@ func (b *saveFail) SaveBlob(_ context.Context, _ restic.BlobType, _ []byte, id r
 	return id, false, 0, nil
 }
 
-func (b *saveFail) Index() restic.MasterIndex {
-	return b.idx
-}
-
 func TestBlobSaver(t *testing.T) {
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 
 	wg, ctx := errgroup.WithContext(ctx)
-	saver := &saveFail{
-		idx: index.NewMasterIndex(),
-	}
+	saver := &saveFail{}
 
 	b := NewBlobSaver(ctx, wg, saver, uint(runtime.NumCPU()))
 
@@ -57,7 +51,7 @@ func TestBlobSaver(t *testing.T) {
 		lock.Lock()
 		results = append(results, SaveBlobResponse{})
 		lock.Unlock()
-		b.Save(ctx, restic.DataBlob, buf, func(res SaveBlobResponse) {
+		b.Save(ctx, restic.DataBlob, buf, "file", func(res SaveBlobResponse) {
 			lock.Lock()
 			results[idx] = res
 			lock.Unlock()
@@ -98,7 +92,6 @@ func TestBlobSaverError(t *testing.T) {
 
 			wg, ctx := errgroup.WithContext(ctx)
 			saver := &saveFail{
-				idx:    index.NewMasterIndex(),
 				failAt: int32(test.failAt),
 			}
 
@@ -106,7 +99,7 @@ func TestBlobSaverError(t *testing.T) {
 
 			for i := 0; i < test.blobs; i++ {
 				buf := &Buffer{Data: []byte(fmt.Sprintf("foo%d", i))}
-				b.Save(ctx, restic.DataBlob, buf, func(res SaveBlobResponse) {})
+				b.Save(ctx, restic.DataBlob, buf, "errfile", func(res SaveBlobResponse) {})
 			}
 
 			b.TriggerShutdown()
@@ -116,9 +109,8 @@ func TestBlobSaverError(t *testing.T) {
 				t.Errorf("expected error not found")
 			}
 
-			if err != errTest {
-				t.Fatalf("unexpected error found: %v", err)
-			}
+			rtest.Assert(t, errors.Is(err, errTest), "unexpected error %v", err)
+			rtest.Assert(t, strings.Contains(err.Error(), "errfile"), "expected error to contain 'errfile' got: %v", err)
 		})
 	}
 }
