@@ -5,8 +5,8 @@ import (
 	"hash"
 	"io"
 
+	"github.com/restic/restic/internal/backend"
 	"github.com/restic/restic/internal/debug"
-	"github.com/restic/restic/internal/restic"
 )
 
 // Backend passes reads through to an underlying layer and accepts writes, but
@@ -15,20 +15,20 @@ import (
 // the repo and does normal operations else.
 // This is used for `backup --dry-run`.
 type Backend struct {
-	b restic.Backend
+	b backend.Backend
 }
 
-// statically ensure that Backend implements restic.Backend.
-var _ restic.Backend = &Backend{}
+// statically ensure that Backend implements backend.Backend.
+var _ backend.Backend = &Backend{}
 
-func New(be restic.Backend) *Backend {
+func New(be backend.Backend) *Backend {
 	b := &Backend{b: be}
 	debug.Log("created new dry backend")
 	return b
 }
 
 // Save adds new Data to the backend.
-func (be *Backend) Save(_ context.Context, h restic.Handle, _ restic.RewindReader) error {
+func (be *Backend) Save(_ context.Context, h backend.Handle, _ backend.RewindReader) error {
 	if err := h.Valid(); err != nil {
 		return err
 	}
@@ -38,7 +38,7 @@ func (be *Backend) Save(_ context.Context, h restic.Handle, _ restic.RewindReade
 }
 
 // Remove deletes a file from the backend.
-func (be *Backend) Remove(_ context.Context, _ restic.Handle) error {
+func (be *Backend) Remove(_ context.Context, _ backend.Handle) error {
 	return nil
 }
 
@@ -46,11 +46,6 @@ func (be *Backend) Connections() uint {
 	return be.b.Connections()
 }
 
-// Location returns the location of the backend.
-func (be *Backend) Location() string {
-	return "DRY:" + be.b.Location()
-}
-
 // Delete removes all data in the backend.
 func (be *Backend) Delete(_ context.Context) error {
 	return nil
@@ -72,14 +67,18 @@ func (be *Backend) IsNotExist(err error) bool {
 	return be.b.IsNotExist(err)
 }
 
-func (be *Backend) List(ctx context.Context, t restic.FileType, fn func(restic.FileInfo) error) error {
+func (be *Backend) IsPermanentError(err error) bool {
+	return be.b.IsPermanentError(err)
+}
+
+func (be *Backend) List(ctx context.Context, t backend.FileType, fn func(backend.FileInfo) error) error {
 	return be.b.List(ctx, t, fn)
 }
 
-func (be *Backend) Load(ctx context.Context, h restic.Handle, length int, offset int64, fn func(io.Reader) error) error {
+func (be *Backend) Load(ctx context.Context, h backend.Handle, length int, offset int64, fn func(io.Reader) error) error {
 	return be.b.Load(ctx, h, length, offset, fn)
 }
 
-func (be *Backend) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, error) {
+func (be *Backend) Stat(ctx context.Context, h backend.Handle) (backend.FileInfo, error) {
 	return be.b.Stat(ctx, h)
 }
