@@ -5,11 +5,13 @@ import (
 	"path/filepath"
 	"testing"
 
-	"github.com/restic/restic/internal/restic"
+	"github.com/restic/restic/internal/backend"
+	"github.com/restic/restic/internal/feature"
 	rtest "github.com/restic/restic/internal/test"
 )
 
 func TestLayout(t *testing.T) {
+	defer feature.TestSetFlag(t, feature.Flag, feature.DeprecateS3LegacyLayout, false)()
 	path := rtest.TempDir(t)
 
 	var tests = []struct {
@@ -49,7 +51,7 @@ func TestLayout(t *testing.T) {
 			}
 
 			packs := make(map[string]bool)
-			err = be.List(context.TODO(), restic.PackFile, func(fi restic.FileInfo) error {
+			err = be.List(context.TODO(), backend.PackFile, func(fi backend.FileInfo) error {
 				packs[fi.Name] = false
 				return nil
 			})
