@@ -5,22 +5,22 @@ import (
 	"hash"
 	"io"
 
+	"github.com/restic/restic/internal/backend"
 	"github.com/restic/restic/internal/errors"
-	"github.com/restic/restic/internal/restic"
 )
 
 // Backend implements a mock backend.
 type Backend struct {
 	CloseFn            func() error
 	IsNotExistFn       func(err error) bool
-	SaveFn             func(ctx context.Context, h restic.Handle, rd restic.RewindReader) error
-	OpenReaderFn       func(ctx context.Context, h restic.Handle, length int, offset int64) (io.ReadCloser, error)
-	StatFn             func(ctx context.Context, h restic.Handle) (restic.FileInfo, error)
-	ListFn             func(ctx context.Context, t restic.FileType, fn func(restic.FileInfo) error) error
-	RemoveFn           func(ctx context.Context, h restic.Handle) error
+	IsPermanentErrorFn func(err error) bool
+	SaveFn             func(ctx context.Context, h backend.Handle, rd backend.RewindReader) error
+	OpenReaderFn       func(ctx context.Context, h backend.Handle, length int, offset int64) (io.ReadCloser, error)
+	StatFn             func(ctx context.Context, h backend.Handle) (backend.FileInfo, error)
+	ListFn             func(ctx context.Context, t backend.FileType, fn func(backend.FileInfo) error) error
+	RemoveFn           func(ctx context.Context, h backend.Handle) error
 	DeleteFn           func(ctx context.Context) error
 	ConnectionsFn      func() uint
-	LocationFn         func() string
 	HasherFn           func() hash.Hash
 	HasAtomicReplaceFn func() bool
 }
@@ -48,15 +48,6 @@ func (m *Backend) Connections() uint {
 	return m.ConnectionsFn()
 }
 
-// Location returns a location string.
-func (m *Backend) Location() string {
-	if m.LocationFn == nil {
-		return ""
-	}
-
-	return m.LocationFn()
-}
-
 // Hasher may return a hash function for calculating a content hash for the backend
 func (m *Backend) Hasher() hash.Hash {
 	if m.HasherFn == nil {
@@ -83,8 +74,16 @@ func (m *Backend) IsNotExist(err error) bool {
 	return m.IsNotExistFn(err)
 }
 
+func (m *Backend) IsPermanentError(err error) bool {
+	if m.IsPermanentErrorFn == nil {
+		return false
+	}
+
+	return m.IsPermanentErrorFn(err)
+}
+
 // Save data in the backend.
-func (m *Backend) Save(ctx context.Context, h restic.Handle, rd restic.RewindReader) error {
+func (m *Backend) Save(ctx context.Context, h backend.Handle, rd backend.RewindReader) error {
 	if m.SaveFn == nil {
 		return errors.New("not implemented")
 	}
@@ -94,7 +93,7 @@ func (m *Backend) Save(ctx context.Context, h restic.Handle, rd restic.RewindRea
 
 // Load runs fn with a reader that yields the contents of the file at h at the
 // given offset.
-func (m *Backend) Load(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
+func (m *Backend) Load(ctx context.Context, h backend.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
 	rd, err := m.openReader(ctx, h, length, offset)
 	if err != nil {
 		return err
@@ -107,7 +106,7 @@ func (m *Backend) Load(ctx context.Context, h restic.Handle, length int, offset
 	return rd.Close()
 }
 
-func (m *Backend) openReader(ctx context.Context, h restic.Handle, length int, offset int64) (io.ReadCloser, error) {
+func (m *Backend) openReader(ctx context.Context, h backend.Handle, length int, offset int64) (io.ReadCloser, error) {
 	if m.OpenReaderFn == nil {
 		return nil, errors.New("not implemented")
 	}
@@ -116,16 +115,16 @@ func (m *Backend) openReader(ctx context.Context, h restic.Handle, length int, o
 }
 
 // Stat an object in the backend.
-func (m *Backend) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, error) {
+func (m *Backend) Stat(ctx context.Context, h backend.Handle) (backend.FileInfo, error) {
 	if m.StatFn == nil {
-		return restic.FileInfo{}, errors.New("not implemented")
+		return backend.FileInfo{}, errors.New("not implemented")
 	}
 
 	return m.StatFn(ctx, h)
 }
 
 // List items of type t.
-func (m *Backend) List(ctx context.Context, t restic.FileType, fn func(restic.FileInfo) error) error {
+func (m *Backend) List(ctx context.Context, t backend.FileType, fn func(backend.FileInfo) error) error {
 	if m.ListFn == nil {
 		return nil
 	}
@@ -134,7 +133,7 @@ func (m *Backend) List(ctx context.Context, t restic.FileType, fn func(restic.Fi
 }
 
 // Remove data from the backend.
-func (m *Backend) Remove(ctx context.Context, h restic.Handle) error {
+func (m *Backend) Remove(ctx context.Context, h backend.Handle) error {
 	if m.RemoveFn == nil {
 		return errors.New("not implemented")
 	}
@@ -152,4 +151,4 @@ func (m *Backend) Delete(ctx context.Context) error {
 }
 
 // Make sure that Backend implements the backend interface.
-var _ restic.Backend = &Backend{}
+var _ backend.Backend = &Backend{}
