@@ -8,20 +8,21 @@ import (
 	"runtime"
 	"sync"
 
+	"github.com/restic/restic/internal/backend"
 	"github.com/restic/restic/internal/errors"
-	"github.com/restic/restic/internal/hashing"
+	"github.com/restic/restic/internal/repository/hashing"
 	"github.com/restic/restic/internal/restic"
 
 	"github.com/restic/restic/internal/crypto"
 	"github.com/restic/restic/internal/debug"
 	"github.com/restic/restic/internal/fs"
-	"github.com/restic/restic/internal/pack"
+	"github.com/restic/restic/internal/repository/pack"
 
 	"github.com/minio/sha256-simd"
 )
 
-// Packer holds a pack.Packer together with a hash writer.
-type Packer struct {
+// packer holds a pack.packer together with a hash writer.
+type packer struct {
 	*pack.Packer
 	tmpfile *os.File
 	bufWr   *bufio.Writer
@@ -31,16 +32,16 @@ type Packer struct {
 type packerManager struct {
 	tpe     restic.BlobType
 	key     *crypto.Key
-	queueFn func(ctx context.Context, t restic.BlobType, p *Packer) error
+	queueFn func(ctx context.Context, t restic.BlobType, p *packer) error
 
 	pm       sync.Mutex
-	packer   *Packer
+	packer   *packer
 	packSize uint
 }
 
-// newPackerManager returns an new packer manager which writes temporary files
+// newPackerManager returns a new packer manager which writes temporary files
 // to a temporary directory
-func newPackerManager(key *crypto.Key, tpe restic.BlobType, packSize uint, queueFn func(ctx context.Context, t restic.BlobType, p *Packer) error) *packerManager {
+func newPackerManager(key *crypto.Key, tpe restic.BlobType, packSize uint, queueFn func(ctx context.Context, t restic.BlobType, p *packer) error) *packerManager {
 	return &packerManager{
 		tpe:      tpe,
 		key:      key,
@@ -113,7 +114,7 @@ func (r *packerManager) SaveBlob(ctx context.Context, t restic.BlobType, id rest
 
 // findPacker returns a packer for a new blob of size bytes. Either a new one is
 // created or one is returned that already has some blobs.
-func (r *packerManager) newPacker() (packer *Packer, err error) {
+func (r *packerManager) newPacker() (pck *packer, err error) {
 	debug.Log("create new pack")
 	tmpfile, err := fs.TempFile("", "restic-temp-pack-")
 	if err != nil {
@@ -122,17 +123,17 @@ func (r *packerManager) newPacker() (packer *Packer, err error) {
 
 	bufWr := bufio.NewWriter(tmpfile)
 	p := pack.NewPacker(r.key, bufWr)
-	packer = &Packer{
+	pck = &packer{
 		Packer:  p,
 		tmpfile: tmpfile,
 		bufWr:   bufWr,
 	}
 
-	return packer, nil
+	return pck, nil
 }
 
 // savePacker stores p in the backend.
-func (r *Repository) savePacker(ctx context.Context, t restic.BlobType, p *Packer) error {
+func (r *Repository) savePacker(ctx context.Context, t restic.BlobType, p *packer) error {
 	debug.Log("save packer for %v with %d blobs (%d bytes)\n", t, p.Packer.Count(), p.Packer.Size())
 	err := p.Packer.Finalize()
 	if err != nil {
@@ -145,7 +146,7 @@ func (r *Repository) savePacker(ctx context.Context, t restic.BlobType, p *Packe
 
 	// calculate sha256 hash in a second pass
 	var rd io.Reader
-	rd, err = restic.NewFileReader(p.tmpfile, nil)
+	rd, err = backend.NewFileReader(p.tmpfile, nil)
 	if err != nil {
 		return err
 	}
@@ -163,12 +164,12 @@ func (r *Repository) savePacker(ctx context.Context, t restic.BlobType, p *Packe
 	}
 
 	id := restic.IDFromHash(hr.Sum(nil))
-	h := restic.Handle{Type: restic.PackFile, Name: id.String(), ContainedBlobType: t}
+	h := backend.Handle{Type: backend.PackFile, Name: id.String(), IsMetadata: t.IsMetadata()}
 	var beHash []byte
 	if beHr != nil {
 		beHash = beHr.Sum(nil)
 	}
-	rrd, err := restic.NewFileReader(p.tmpfile, beHash)
+	rrd, err := backend.NewFileReader(p.tmpfile, beHash)
 	if err != nil {
 		return err
 	}
@@ -199,8 +200,5 @@ func (r *Repository) savePacker(ctx context.Context, t restic.BlobType, p *Packe
 	r.idx.StorePack(id, p.Packer.Blobs())
 
 	// Save index if full
-	if r.noAutoIndexUpdate {
-		return nil
-	}
 	return r.idx.SaveFullIndex(ctx, r)
 }
