@@ -11,7 +11,7 @@ import (
 // LoadJSONUnpacked decrypts the data and afterwards calls json.Unmarshal on
 // the item.
 func LoadJSONUnpacked(ctx context.Context, repo LoaderUnpacked, t FileType, id ID, item interface{}) (err error) {
-	buf, err := repo.LoadUnpacked(ctx, t, id, nil)
+	buf, err := repo.LoadUnpacked(ctx, t, id)
 	if err != nil {
 		return err
 	}
