@@ -13,6 +13,7 @@ import (
 	"time"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/restic/restic/internal/errors"
 	"github.com/restic/restic/internal/test"
 	rtest "github.com/restic/restic/internal/test"
 )
@@ -196,6 +197,20 @@ var nodeTests = []Node{
 			{"user.foo", []byte("bar")},
 		},
 	},
+	{
+		Name:       "testXattrFileMacOSResourceFork",
+		Type:       "file",
+		Content:    IDs{},
+		UID:        uint32(os.Getuid()),
+		GID:        uint32(os.Getgid()),
+		Mode:       0604,
+		ModTime:    parseTime("2005-05-14 21:07:03.111"),
+		AccessTime: parseTime("2005-05-14 21:07:04.222"),
+		ChangeTime: parseTime("2005-05-14 21:07:05.333"),
+		ExtendedAttributes: []ExtendedAttribute{
+			{"com.apple.ResourceFork", []byte("bar")},
+		},
+	},
 }
 
 func TestNodeRestoreAt(t *testing.T) {
@@ -215,6 +230,11 @@ func TestNodeRestoreAt(t *testing.T) {
 						extAttrArr[i].Name = strings.ToUpper(extAttrArr[i].Name)
 					}
 				}
+				for _, attr := range test.ExtendedAttributes {
+					if strings.HasPrefix(attr.Name, "com.apple.") && runtime.GOOS != "darwin" {
+						t.Skipf("attr %v only relevant on macOS", attr.Name)
+					}
+				}
 
 				// tempdir might be backed by a filesystem that does not support
 				// extended attributes
@@ -228,10 +248,6 @@ func TestNodeRestoreAt(t *testing.T) {
 			rtest.OK(t, test.CreateAt(context.TODO(), nodePath, nil))
 			rtest.OK(t, test.RestoreMetadata(nodePath, func(msg string) { rtest.OK(t, fmt.Errorf("Warning triggered for path: %s: %s", nodePath, msg)) }))
 
-			if test.Type == "dir" {
-				rtest.OK(t, test.RestoreTimestamps(nodePath))
-			}
-
 			fi, err := os.Lstat(nodePath)
 			rtest.OK(t, err)
 
@@ -382,3 +398,14 @@ func TestSymlinkSerializationFormat(t *testing.T) {
 		test.Assert(t, n2.LinkTargetRaw == nil, "quoted link target is just a helper field and must be unset after decoding")
 	}
 }
+
+func TestNodeRestoreMetadataError(t *testing.T) {
+	tempdir := t.TempDir()
+
+	node := nodeTests[0]
+	nodePath := filepath.Join(tempdir, node.Name)
+
+	// This will fail because the target file does not exist
+	err := node.RestoreMetadata(nodePath, func(msg string) { rtest.OK(t, fmt.Errorf("Warning triggered for path: %s: %s", nodePath, msg)) })
+	test.Assert(t, errors.Is(err, os.ErrNotExist), "failed for an unexpected reason")
+}
