@@ -4,13 +4,11 @@ import (
 	"bytes"
 	"context"
 	"fmt"
-	"io"
 	"os"
+	"sort"
 	"testing"
 
-	"github.com/restic/restic/internal/crypto"
 	"github.com/restic/restic/internal/errors"
-	"github.com/restic/restic/internal/repository"
 	"github.com/restic/restic/internal/restic"
 	rtest "github.com/restic/restic/internal/test"
 )
@@ -26,11 +24,6 @@ type TestFile struct {
 }
 
 type TestRepo struct {
-	key *crypto.Key
-
-	// pack names and ids
-	packsNameToID map[string]restic.ID
-	packsIDToName map[restic.ID]string
 	packsIDToData map[restic.ID][]byte
 
 	// blobs and files
@@ -39,11 +32,11 @@ type TestRepo struct {
 	filesPathToContent map[string]string
 
 	//
-	loader repository.BackendLoadFn
+	loader blobsLoaderFn
 }
 
-func (i *TestRepo) Lookup(bh restic.BlobHandle) []restic.PackedBlob {
-	packs := i.blobs[bh.ID]
+func (i *TestRepo) Lookup(tpe restic.BlobType, id restic.ID) []restic.PackedBlob {
+	packs := i.blobs[id]
 	return packs
 }
 
@@ -58,16 +51,6 @@ func newTestRepo(content []TestFile) *TestRepo {
 		blobs map[restic.ID]restic.Blob
 	}
 	packs := make(map[string]Pack)
-
-	key := crypto.NewRandomKey()
-	seal := func(data []byte) []byte {
-		ciphertext := crypto.NewBlobBuffer(len(data))
-		ciphertext = ciphertext[:0] // truncate the slice
-		nonce := crypto.NewRandomNonce()
-		ciphertext = append(ciphertext, nonce...)
-		return key.Seal(ciphertext, nonce, data, nil)
-	}
-
 	filesPathToContent := make(map[string]string)
 
 	for _, file := range content {
@@ -85,14 +68,15 @@ func newTestRepo(content []TestFile) *TestRepo {
 			// calculate blob id and add to the pack as necessary
 			blobID := restic.Hash([]byte(blob.data))
 			if _, found := pack.blobs[blobID]; !found {
-				blobData := seal([]byte(blob.data))
+				blobData := []byte(blob.data)
 				pack.blobs[blobID] = restic.Blob{
 					BlobHandle: restic.BlobHandle{
 						Type: restic.DataBlob,
 						ID:   blobID,
 					},
-					Length: uint(len(blobData)),
-					Offset: uint(len(pack.data)),
+					Length:             uint(len(blobData)),
+					UncompressedLength: uint(len(blobData)),
+					Offset:             uint(len(pack.data)),
 				}
 				pack.data = append(pack.data, blobData...)
 			}
@@ -103,15 +87,11 @@ func newTestRepo(content []TestFile) *TestRepo {
 	}
 
 	blobs := make(map[restic.ID][]restic.PackedBlob)
-	packsIDToName := make(map[restic.ID]string)
 	packsIDToData := make(map[restic.ID][]byte)
-	packsNameToID := make(map[string]restic.ID)
 
 	for _, pack := range packs {
 		packID := restic.Hash(pack.data)
-		packsIDToName[packID] = pack.name
 		packsIDToData[packID] = pack.data
-		packsNameToID[pack.name] = packID
 		for blobID, blob := range pack.blobs {
 			blobs[blobID] = append(blobs[blobID], restic.PackedBlob{Blob: blob, PackID: packID})
 		}
@@ -127,30 +107,44 @@ func newTestRepo(content []TestFile) *TestRepo {
 	}
 
 	repo := &TestRepo{
-		key:                key,
-		packsIDToName:      packsIDToName,
 		packsIDToData:      packsIDToData,
-		packsNameToID:      packsNameToID,
 		blobs:              blobs,
 		files:              files,
 		filesPathToContent: filesPathToContent,
 	}
-	repo.loader = func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
-		packID, err := restic.ParseID(h.Name)
-		if err != nil {
-			return err
+	repo.loader = func(ctx context.Context, packID restic.ID, blobs []restic.Blob, handleBlobFn func(blob restic.BlobHandle, buf []byte, err error) error) error {
+		blobs = append([]restic.Blob{}, blobs...)
+		sort.Slice(blobs, func(i, j int) bool {
+			return blobs[i].Offset < blobs[j].Offset
+		})
+
+		for _, blob := range blobs {
+			found := false
+			for _, e := range repo.blobs[blob.ID] {
+				if packID == e.PackID {
+					found = true
+					buf := repo.packsIDToData[packID][e.Offset : e.Offset+e.Length]
+					err := handleBlobFn(e.BlobHandle, buf, nil)
+					if err != nil {
+						return err
+					}
+				}
+			}
+			if !found {
+				return fmt.Errorf("missing blob: %v", blob)
+			}
 		}
-		rd := bytes.NewReader(repo.packsIDToData[packID][int(offset) : int(offset)+length])
-		return fn(rd)
+		return nil
 	}
 
 	return repo
 }
 
 func restoreAndVerify(t *testing.T, tempdir string, content []TestFile, files map[string]bool, sparse bool) {
+	t.Helper()
 	repo := newTestRepo(content)
 
-	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup, 2, sparse, nil)
+	r := newFileRestorer(tempdir, repo.loader, repo.Lookup, 2, sparse, false, nil)
 
 	if files == nil {
 		r.files = repo.files
@@ -169,6 +163,7 @@ func restoreAndVerify(t *testing.T, tempdir string, content []TestFile, files ma
 }
 
 func verifyRestore(t *testing.T, r *fileRestorer, repo *TestRepo) {
+	t.Helper()
 	for _, file := range r.files {
 		target := r.targetPath(file.location)
 		data, err := os.ReadFile(target)
@@ -211,6 +206,10 @@ func TestFileRestorerBasic(t *testing.T) {
 					{"data3-1", "pack3-1"},
 				},
 			},
+			{
+				name:  "empty",
+				blobs: []TestBlob{},
+			},
 		}, nil, sparse)
 	}
 }
@@ -282,62 +281,17 @@ func TestErrorRestoreFiles(t *testing.T) {
 
 	loadError := errors.New("load error")
 	// loader always returns an error
-	repo.loader = func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
+	repo.loader = func(ctx context.Context, packID restic.ID, blobs []restic.Blob, handleBlobFn func(blob restic.BlobHandle, buf []byte, err error) error) error {
 		return loadError
 	}
 
-	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup, 2, false, nil)
+	r := newFileRestorer(tempdir, repo.loader, repo.Lookup, 2, false, false, nil)
 	r.files = repo.files
 
 	err := r.restoreFiles(context.TODO())
 	rtest.Assert(t, errors.Is(err, loadError), "got %v, expected contained error %v", err, loadError)
 }
 
-func TestDownloadError(t *testing.T) {
-	for i := 0; i < 100; i += 10 {
-		testPartialDownloadError(t, i)
-	}
-}
-
-func testPartialDownloadError(t *testing.T, part int) {
-	tempdir := rtest.TempDir(t)
-	content := []TestFile{
-		{
-			name: "file1",
-			blobs: []TestBlob{
-				{"data1-1", "pack1"},
-				{"data1-2", "pack1"},
-				{"data1-3", "pack1"},
-			},
-		}}
-
-	repo := newTestRepo(content)
-
-	// loader always returns an error
-	loader := repo.loader
-	repo.loader = func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
-		// only load partial data to execise fault handling in different places
-		err := loader(ctx, h, length*part/100, offset, fn)
-		if err == nil {
-			return nil
-		}
-		fmt.Println("Retry after error", err)
-		return loader(ctx, h, length, offset, fn)
-	}
-
-	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup, 2, false, nil)
-	r.files = repo.files
-	r.Error = func(s string, e error) error {
-		// ignore errors as in the `restore` command
-		fmt.Println("error during restore", s, e)
-		return nil
-	}
-
-	err := r.restoreFiles(context.TODO())
-	rtest.OK(t, err)
-	verifyRestore(t, r, repo)
-}
-
 func TestFatalDownloadError(t *testing.T) {
 	tempdir := rtest.TempDir(t)
 	content := []TestFile{
@@ -360,12 +314,19 @@ func TestFatalDownloadError(t *testing.T) {
 	repo := newTestRepo(content)
 
 	loader := repo.loader
-	repo.loader = func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
-		// only return half the data to break file2
-		return loader(ctx, h, length/2, offset, fn)
+	repo.loader = func(ctx context.Context, packID restic.ID, blobs []restic.Blob, handleBlobFn func(blob restic.BlobHandle, buf []byte, err error) error) error {
+		ctr := 0
+		return loader(ctx, packID, blobs, func(blob restic.BlobHandle, buf []byte, err error) error {
+			if ctr < 2 {
+				ctr++
+				return handleBlobFn(blob, buf, err)
+			}
+			// break file2
+			return errors.New("failed to load blob")
+		})
 	}
 
-	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup, 2, false, nil)
+	r := newFileRestorer(tempdir, repo.loader, repo.Lookup, 2, false, false, nil)
 	r.files = repo.files
 
 	var errors []string
