@@ -3,7 +3,9 @@ package test
 import (
 	"compress/bzip2"
 	"compress/gzip"
+	"fmt"
 	"io"
+	"math/rand"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -11,8 +13,6 @@ import (
 	"testing"
 
 	"github.com/restic/restic/internal/errors"
-
-	mrand "math/rand"
 )
 
 // Assert fails the test if the condition is false.
@@ -47,10 +47,22 @@ func OKs(tb testing.TB, errs []error) {
 }
 
 // Equals fails the test if exp is not equal to act.
-func Equals(tb testing.TB, exp, act interface{}) {
+// msg is optional message to be printed, first param being format string and rest being arguments.
+func Equals(tb testing.TB, exp, act interface{}, msgs ...string) {
 	tb.Helper()
 	if !reflect.DeepEqual(exp, act) {
-		tb.Fatalf("\033[31m\n\n\texp: %#v\n\n\tgot: %#v\033[39m\n\n", exp, act)
+		var msgString string
+		length := len(msgs)
+		if length == 1 {
+			msgString = msgs[0]
+		} else if length > 1 {
+			args := make([]interface{}, length-1)
+			for i, msg := range msgs[1:] {
+				args[i] = msg
+			}
+			msgString = fmt.Sprintf(msgs[0], args...)
+		}
+		tb.Fatalf("\033[31m\n\n\t"+msgString+"\n\n\texp: %#v\n\n\tgot: %#v\033[39m\n\n", exp, act)
 	}
 }
 
@@ -58,7 +70,7 @@ func Equals(tb testing.TB, exp, act interface{}) {
 func Random(seed, count int) []byte {
 	p := make([]byte, count)
 
-	rnd := mrand.New(mrand.NewSource(int64(seed)))
+	rnd := rand.New(rand.NewSource(int64(seed)))
 
 	for i := 0; i < len(p); i += 8 {
 		val := rnd.Int63()
