@@ -7,14 +7,13 @@ import (
 	"github.com/restic/restic/internal/archiver"
 	"github.com/restic/restic/internal/restic"
 	"github.com/restic/restic/internal/ui"
-	"github.com/restic/restic/internal/ui/termstatus"
 )
 
 // JSONProgress reports progress for the `backup` command in JSON.
 type JSONProgress struct {
 	*ui.Message
 
-	term *termstatus.Terminal
+	term ui.Terminal
 	v    uint
 }
 
@@ -22,7 +21,7 @@ type JSONProgress struct {
 var _ ProgressPrinter = &JSONProgress{}
 
 // NewJSONProgress returns a new backup progress reporter.
-func NewJSONProgress(term *termstatus.Terminal, verbosity uint) *JSONProgress {
+func NewJSONProgress(term ui.Terminal, verbosity uint) *JSONProgress {
 	return &JSONProgress{
 		Message: ui.NewMessage(term, verbosity),
 		term:    term,
@@ -68,7 +67,7 @@ func (b *JSONProgress) Update(total, processed Counter, errors uint, currentFile
 func (b *JSONProgress) ScannerError(item string, err error) error {
 	b.error(errorUpdate{
 		MessageType: "error",
-		Error:       err,
+		Error:       errorObject{err.Error()},
 		During:      "scan",
 		Item:        item,
 	})
@@ -79,7 +78,7 @@ func (b *JSONProgress) ScannerError(item string, err error) error {
 func (b *JSONProgress) Error(item string, err error) error {
 	b.error(errorUpdate{
 		MessageType: "error",
-		Error:       err,
+		Error:       errorObject{err.Error()},
 		During:      "archival",
 		Item:        item,
 	})
@@ -206,11 +205,15 @@ type statusUpdate struct {
 	CurrentFiles     []string `json:"current_files,omitempty"`
 }
 
+type errorObject struct {
+	Message string `json:"message"`
+}
+
 type errorUpdate struct {
-	MessageType string `json:"message_type"` // "error"
-	Error       error  `json:"error"`
-	During      string `json:"during"`
-	Item        string `json:"item"`
+	MessageType string      `json:"message_type"` // "error"
+	Error       errorObject `json:"error"`
+	During      string      `json:"during"`
+	Item        string      `json:"item"`
 }
 
 type verboseUpdate struct {
