@@ -13,7 +13,7 @@ import (
 // TestTree is used to construct a list of trees for testing the walker.
 type TestTree map[string]interface{}
 
-// TestNode is used to test the walker.
+// TestFile is used to test the walker.
 type TestFile struct {
 	Size uint64
 }
@@ -93,28 +93,32 @@ func (t TreeMap) Connections() uint {
 
 // checkFunc returns a function suitable for walking the tree to check
 // something, and a function which will check the final result.
-type checkFunc func(t testing.TB) (walker WalkFunc, final func(testing.TB))
+type checkFunc func(t testing.TB) (walker WalkFunc, leaveDir func(path string), final func(testing.TB))
 
 // checkItemOrder ensures that the order of the 'path' arguments is the one passed in as 'want'.
 func checkItemOrder(want []string) checkFunc {
 	pos := 0
-	return func(t testing.TB) (walker WalkFunc, final func(testing.TB)) {
-		walker = func(treeID restic.ID, path string, node *restic.Node, err error) (bool, error) {
+	return func(t testing.TB) (walker WalkFunc, leaveDir func(path string), final func(testing.TB)) {
+		walker = func(treeID restic.ID, path string, node *restic.Node, err error) error {
 			if err != nil {
 				t.Errorf("error walking %v: %v", path, err)
-				return false, err
+				return err
 			}
 
 			if pos >= len(want) {
 				t.Errorf("additional unexpected path found: %v", path)
-				return false, nil
+				return nil
 			}
 
 			if path != want[pos] {
 				t.Errorf("wrong path found, want %q, got %q", want[pos], path)
 			}
 			pos++
-			return false, nil
+			return nil
+		}
+
+		leaveDir = func(path string) {
+			_ = walker(restic.ID{}, "leave: "+path, nil, nil)
 		}
 
 		final = func(t testing.TB) {
@@ -123,30 +127,30 @@ func checkItemOrder(want []string) checkFunc {
 			}
 		}
 
-		return walker, final
+		return walker, leaveDir, final
 	}
 }
 
 // checkParentTreeOrder ensures that the order of the 'parentID' arguments is the one passed in as 'want'.
 func checkParentTreeOrder(want []string) checkFunc {
 	pos := 0
-	return func(t testing.TB) (walker WalkFunc, final func(testing.TB)) {
-		walker = func(treeID restic.ID, path string, node *restic.Node, err error) (bool, error) {
+	return func(t testing.TB) (walker WalkFunc, leaveDir func(path string), final func(testing.TB)) {
+		walker = func(treeID restic.ID, path string, node *restic.Node, err error) error {
 			if err != nil {
 				t.Errorf("error walking %v: %v", path, err)
-				return false, err
+				return err
 			}
 
 			if pos >= len(want) {
 				t.Errorf("additional unexpected parent tree ID found: %v", treeID)
-				return false, nil
+				return nil
 			}
 
 			if treeID.String() != want[pos] {
 				t.Errorf("wrong parent tree ID found, want %q, got %q", want[pos], treeID.String())
 			}
 			pos++
-			return false, nil
+			return nil
 		}
 
 		final = func(t testing.TB) {
@@ -155,7 +159,7 @@ func checkParentTreeOrder(want []string) checkFunc {
 			}
 		}
 
-		return walker, final
+		return walker, nil, final
 	}
 }
 
@@ -164,16 +168,16 @@ func checkParentTreeOrder(want []string) checkFunc {
 func checkSkipFor(skipFor map[string]struct{}, wantPaths []string) checkFunc {
 	var pos int
 
-	return func(t testing.TB) (walker WalkFunc, final func(testing.TB)) {
-		walker = func(treeID restic.ID, path string, node *restic.Node, err error) (bool, error) {
+	return func(t testing.TB) (walker WalkFunc, leaveDir func(path string), final func(testing.TB)) {
+		walker = func(treeID restic.ID, path string, node *restic.Node, err error) error {
 			if err != nil {
 				t.Errorf("error walking %v: %v", path, err)
-				return false, err
+				return err
 			}
 
 			if pos >= len(wantPaths) {
 				t.Errorf("additional unexpected path found: %v", path)
-				return false, nil
+				return nil
 			}
 
 			if path != wantPaths[pos] {
@@ -182,10 +186,14 @@ func checkSkipFor(skipFor map[string]struct{}, wantPaths []string) checkFunc {
 			pos++
 
 			if _, ok := skipFor[path]; ok {
-				return false, ErrSkipNode
+				return ErrSkipNode
 			}
 
-			return false, nil
+			return nil
+		}
+
+		leaveDir = func(path string) {
+			_ = walker(restic.ID{}, "leave: "+path, nil, nil)
 		}
 
 		final = func(t testing.TB) {
@@ -194,47 +202,7 @@ func checkSkipFor(skipFor map[string]struct{}, wantPaths []string) checkFunc {
 			}
 		}
 
-		return walker, final
-	}
-}
-
-// checkIgnore returns ErrSkipNode if path is in skipFor and sets ignore according
-// to ignoreFor. It checks that the paths the walk func is called for are exactly
-// the ones in wantPaths.
-func checkIgnore(skipFor map[string]struct{}, ignoreFor map[string]bool, wantPaths []string) checkFunc {
-	var pos int
-
-	return func(t testing.TB) (walker WalkFunc, final func(testing.TB)) {
-		walker = func(treeID restic.ID, path string, node *restic.Node, err error) (bool, error) {
-			if err != nil {
-				t.Errorf("error walking %v: %v", path, err)
-				return false, err
-			}
-
-			if pos >= len(wantPaths) {
-				t.Errorf("additional unexpected path found: %v", path)
-				return ignoreFor[path], nil
-			}
-
-			if path != wantPaths[pos] {
-				t.Errorf("wrong path found, want %q, got %q", wantPaths[pos], path)
-			}
-			pos++
-
-			if _, ok := skipFor[path]; ok {
-				return ignoreFor[path], ErrSkipNode
-			}
-
-			return ignoreFor[path], nil
-		}
-
-		final = func(t testing.TB) {
-			if pos != len(wantPaths) {
-				t.Errorf("wrong number of paths returned, want %d, got %d", len(wantPaths), pos)
-			}
-		}
-
-		return walker, final
+		return walker, leaveDir, final
 	}
 }
 
@@ -256,6 +224,8 @@ func TestWalker(t *testing.T) {
 					"/foo",
 					"/subdir",
 					"/subdir/subfile",
+					"leave: /subdir",
+					"leave: /",
 				}),
 				checkParentTreeOrder([]string{
 					"a760536a8fd64dd63f8dd95d85d788d71fd1bee6828619350daf6959dcb499a0", // tree /
@@ -270,16 +240,14 @@ func TestWalker(t *testing.T) {
 						"/",
 						"/foo",
 						"/subdir",
+						"leave: /",
 					},
 				),
-				checkIgnore(
-					map[string]struct{}{}, map[string]bool{
-						"/subdir": true,
+				checkSkipFor(
+					map[string]struct{}{
+						"/": {},
 					}, []string{
 						"/",
-						"/foo",
-						"/subdir",
-						"/subdir/subfile",
 					},
 				),
 			},
@@ -303,10 +271,14 @@ func TestWalker(t *testing.T) {
 					"/foo",
 					"/subdir1",
 					"/subdir1/subfile1",
+					"leave: /subdir1",
 					"/subdir2",
 					"/subdir2/subfile2",
 					"/subdir2/subsubdir2",
 					"/subdir2/subsubdir2/subsubfile3",
+					"leave: /subdir2/subsubdir2",
+					"leave: /subdir2",
+					"leave: /",
 				}),
 				checkParentTreeOrder([]string{
 					"7a0e59b986cc83167d9fbeeefc54e4629770124c5825d391f7ee0598667fcdf1", // tree /
@@ -329,6 +301,9 @@ func TestWalker(t *testing.T) {
 						"/subdir2/subfile2",
 						"/subdir2/subsubdir2",
 						"/subdir2/subsubdir2/subsubfile3",
+						"leave: /subdir2/subsubdir2",
+						"leave: /subdir2",
+						"leave: /",
 					},
 				),
 				checkSkipFor(
@@ -342,6 +317,8 @@ func TestWalker(t *testing.T) {
 						"/subdir2",
 						"/subdir2/subfile2",
 						"/subdir2/subsubdir2",
+						"leave: /subdir2",
+						"leave: /",
 					},
 				),
 				checkSkipFor(
@@ -350,6 +327,7 @@ func TestWalker(t *testing.T) {
 					}, []string{
 						"/",
 						"/foo",
+						"leave: /",
 					},
 				),
 			},
@@ -382,15 +360,19 @@ func TestWalker(t *testing.T) {
 					"/subdir1/subfile1",
 					"/subdir1/subfile2",
 					"/subdir1/subfile3",
+					"leave: /subdir1",
 					"/subdir2",
 					"/subdir2/subfile1",
 					"/subdir2/subfile2",
 					"/subdir2/subfile3",
+					"leave: /subdir2",
 					"/subdir3",
 					"/subdir3/subfile1",
 					"/subdir3/subfile2",
 					"/subdir3/subfile3",
+					"leave: /subdir3",
 					"/zzz other",
+					"leave: /",
 				}),
 				checkParentTreeOrder([]string{
 					"c2efeff7f217a4dfa12a16e8bb3cefedd37c00873605c29e5271c6061030672f", // tree /
@@ -409,81 +391,6 @@ func TestWalker(t *testing.T) {
 					"57ee8960c7a86859b090a76e5d013f83d10c0ce11d5460076ca8468706f784ab", // tree /subdir3
 					"c2efeff7f217a4dfa12a16e8bb3cefedd37c00873605c29e5271c6061030672f", // tree /
 				}),
-				checkIgnore(
-					map[string]struct{}{
-						"/subdir1": {},
-					}, map[string]bool{
-						"/subdir1": true,
-					}, []string{
-						"/",
-						"/foo",
-						"/subdir1",
-						"/zzz other",
-					},
-				),
-				checkIgnore(
-					map[string]struct{}{}, map[string]bool{
-						"/subdir1": true,
-					}, []string{
-						"/",
-						"/foo",
-						"/subdir1",
-						"/subdir1/subfile1",
-						"/subdir1/subfile2",
-						"/subdir1/subfile3",
-						"/zzz other",
-					},
-				),
-				checkIgnore(
-					map[string]struct{}{
-						"/subdir2": {},
-					}, map[string]bool{
-						"/subdir2": true,
-					}, []string{
-						"/",
-						"/foo",
-						"/subdir1",
-						"/subdir1/subfile1",
-						"/subdir1/subfile2",
-						"/subdir1/subfile3",
-						"/subdir2",
-						"/zzz other",
-					},
-				),
-				checkIgnore(
-					map[string]struct{}{}, map[string]bool{
-						"/subdir1/subfile1": true,
-						"/subdir1/subfile2": true,
-						"/subdir1/subfile3": true,
-					}, []string{
-						"/",
-						"/foo",
-						"/subdir1",
-						"/subdir1/subfile1",
-						"/subdir1/subfile2",
-						"/subdir1/subfile3",
-						"/zzz other",
-					},
-				),
-				checkIgnore(
-					map[string]struct{}{}, map[string]bool{
-						"/subdir2/subfile1": true,
-						"/subdir2/subfile2": true,
-						"/subdir2/subfile3": true,
-					}, []string{
-						"/",
-						"/foo",
-						"/subdir1",
-						"/subdir1/subfile1",
-						"/subdir1/subfile2",
-						"/subdir1/subfile3",
-						"/subdir2",
-						"/subdir2/subfile1",
-						"/subdir2/subfile2",
-						"/subdir2/subfile3",
-						"/zzz other",
-					},
-				),
 			},
 		},
 		{
@@ -503,45 +410,23 @@ func TestWalker(t *testing.T) {
 				checkItemOrder([]string{
 					"/",
 					"/subdir1",
+					"leave: /subdir1",
 					"/subdir2",
+					"leave: /subdir2",
 					"/subdir3",
 					"/subdir3/file",
+					"leave: /subdir3",
 					"/subdir4",
 					"/subdir4/file",
+					"leave: /subdir4",
 					"/subdir5",
+					"leave: /subdir5",
 					"/subdir6",
+					"leave: /subdir6",
+					"leave: /",
 				}),
 			},
 		},
-		{
-			tree: TestTree{
-				"subdir1": TestTree{},
-				"subdir2": TestTree{},
-				"subdir3": TestTree{
-					"file": TestFile{},
-				},
-				"subdir4": TestTree{},
-				"subdir5": TestTree{
-					"file": TestFile{},
-				},
-				"subdir6": TestTree{},
-			},
-			checks: []checkFunc{
-				checkIgnore(
-					map[string]struct{}{}, map[string]bool{
-						"/subdir2": true,
-					}, []string{
-						"/",
-						"/subdir1",
-						"/subdir2",
-						"/subdir3",
-						"/subdir3/file",
-						"/subdir5",
-						"/subdir5/file",
-					},
-				),
-			},
-		},
 	}
 
 	for _, test := range tests {
@@ -552,8 +437,11 @@ func TestWalker(t *testing.T) {
 					ctx, cancel := context.WithCancel(context.TODO())
 					defer cancel()
 
-					fn, last := check(t)
-					err := Walk(ctx, repo, root, restic.NewIDSet(), fn)
+					fn, leaveDir, last := check(t)
+					err := Walk(ctx, repo, root, WalkVisitor{
+						ProcessNode: fn,
+						LeaveDir:    leaveDir,
+					})
 					if err != nil {
 						t.Error(err)
 					}
